import { Component, OnDestroy, OnInit } from '@angular/core';
import { Page } from '../../../core/models/page';
import { ArticlePrice } from '../../../core/models/price';
import { PriceService } from '../price.service';
import { debounceTime, distinctUntilChanged, Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { ChangeDetectorRef } from '@angular/core';
import { NzMessageModule, NzMessageService } from 'ng-zorro-antd/message';
import { NzModalModule, NzModalService } from 'ng-zorro-antd/modal';
import { NzFormModule } from 'ng-zorro-antd/form';
import { NzInputModule } from 'ng-zorro-antd/input';
import { NzInputNumberModule } from 'ng-zorro-antd/input-number';
import { NzCardModule } from 'ng-zorro-antd/card';
import { NzTableModule } from 'ng-zorro-antd/table';
import { NzIconModule } from 'ng-zorro-antd/icon';
import { FormsModule } from '@angular/forms';
import { FormBuilder, FormGroup, Validators, ReactiveFormsModule } from '@angular/forms';
import { NgForOf } from "@angular/common";
import { SetArticlePriceComponent } from "./set-article-price/set-article-price.component";

@Component({
  selector: 'app-article',
  standalone: true,
  imports: [
    NzMessageModule,
    NzModalModule,
    NzFormModule,
    NzInputModule,
    NzInputNumberModule,
    NzCardModule,
    NzTableModule,
    NzIconModule,
    FormsModule,
    ReactiveFormsModule,
    NgForOf,
    SetArticlePriceComponent
  ],
  templateUrl: './article.component.html',
  styleUrl: './article.component.scss'
})
export class ArticleComponent implements OnInit, OnDestroy {
  pageData: Page<ArticlePrice> = new Page<ArticlePrice>();
  searchParams: { [param: string]: any } = {
    page: 0,
    size: 10,
    search_like_username: ''
  };
  onValueChange = new Subject<string>();
  loading = false;
  destroy$ = new Subject();
  editForm: FormGroup;

  constructor(
    private priceService: PriceService,
    private cdr: ChangeDetectorRef,
    private message: NzMessageService,
    private modal: NzModalService,
    private fb: FormBuilder
  ) {
    this.onValueChange.pipe(
      debounceTime(300),
      distinctUntilChanged(),
      takeUntil(this.destroy$)
    ).subscribe(() => {
      this.searchParams['page'] = 0;
      this.cdr.markForCheck();
      this.loadData();
    });

    this.editForm = this.fb.group({
      price: [null, [Validators.required, Validators.min(0)]],
      discount: [1, [Validators.required, Validators.min(0), Validators.max(1)]]
    });
  }

  ngOnInit(): void {
    console.log('Component initialized');
    this.onValueChange
      .pipe(debounceTime(500))
      .subscribe(() => {
        this.loadData();
      });
    this.loadData();
  }

  ngOnDestroy(): void {
    this.destroy$.next(null);
    this.destroy$.complete();
  }

  loadData() {
    console.log('Loading data with params:', this.searchParams);
    this.loading = true;
    this.cdr.detectChanges();

    this.priceService.pageOfArticle(this.searchParams)
      .pipe(takeUntil(this.destroy$))
      .subscribe({
        next: data => {
          console.log('Raw response data:', data);
          this.pageData = data;
          console.log('Processed pageData:', this.pageData);
          if (this.pageData?.items?.length > 0) {
            console.log('First item:', this.pageData.items[0]);
            console.log('First item article:', this.pageData.items[0]?.article);
            console.log('First item price:', this.pageData.items[0]?.price);
          }
          this.loading = false;
          this.cdr.detectChanges();
        },
        error: error => {
          console.error('Error loading data:', error);
          this.loading = false;
          this.message.error("获取文章数据失败！");
          this.cdr.detectChanges();
        }
      });
  }

  changePaginationPage(page: number): void {
    console.log('Changing page to:', page);
    this.searchParams['page'] = page - 1;
    this.cdr.detectChanges();
    this.loadData();
  }

  changePaginationSize(size: number): void {
    console.log('Changing page size to:', size);
    this.searchParams['page'] = 0;
    this.searchParams['size'] = size;
    this.cdr.detectChanges();
    this.loadData();
  }

  editPrice(article: ArticlePrice) {
    console.log('Editing article:', article);
    const modalRef = this.modal.create<SetArticlePriceComponent>({
      nzTitle: '编辑文章价格',
      nzContent: SetArticlePriceComponent,
      nzData: article,
      nzFooter: null
    });

    modalRef.afterClose.subscribe(result => {
      if (result) {
        this.loadData();
      }
    });
  }

  // 将元转换为分
  private convertYuanToCents(yuan: number): number {
    return Math.round(yuan * 100);
  }
}
