import { Component, OnDestroy, OnInit } from '@angular/core';
import { Page } from '../../../core/models/page';
import { ColumnPrice } from '../../../core/models/price';
import { PriceService } from '../price.service';
import { debounceTime, distinctUntilChanged, Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { ChangeDetectorRef } from '@angular/core';
import { NzMessageModule, NzMessageService } from 'ng-zorro-antd/message';
import { NzModalModule, NzModalService } from 'ng-zorro-antd/modal';
import { NzFormModule } from 'ng-zorro-antd/form';
import { NzInputModule } from 'ng-zorro-antd/input';
import { NzInputNumberModule } from 'ng-zorro-antd/input-number';
import { NzCardModule } from 'ng-zorro-antd/card';
import { NzTableModule } from 'ng-zorro-antd/table';
import { NzIconModule } from 'ng-zorro-antd/icon';
import { FormsModule } from '@angular/forms';
import { FormBuilder, FormGroup, Validators, ReactiveFormsModule } from '@angular/forms';
import {SetColumnPriceComponent} from "./set-column-price/set-column-price.component";
import {NgForOf} from "@angular/common";

@Component({
  selector: 'app-column',
  standalone: true,
  imports: [
    NzMessageModule,
    NzModalModule,
    NzFormModule,
    NzInputModule,
    NzInputNumberModule,
    NzCardModule,
    NzTableModule,
    NzIconModule,
    FormsModule,
    ReactiveFormsModule,
    NgForOf
  ],
  templateUrl: './column.component.html',
  styleUrl: './column.component.scss'
})
export class ColumnComponent implements OnInit, OnDestroy {
  pageData: Page<ColumnPrice> = new Page<ColumnPrice>();
  searchParams: { [param: string]: any } = {
    page: 0,
    size: 10,
    search_like_username: ''
  };
  onValueChange = new Subject<string>();
  loading = false;
  destroy$ = new Subject();
  editForm: FormGroup;

  constructor(
    private priceService: PriceService,
    private cdr: ChangeDetectorRef,
    private message: NzMessageService,
    private modal: NzModalService,
    private fb: FormBuilder
  ) {
    this.onValueChange.pipe(
      debounceTime(300),
      distinctUntilChanged(),
      takeUntil(this.destroy$)
    ).subscribe(() => {
      this.searchParams['page'] = 0;
      this.cdr.markForCheck();
      this.loadData();
    });

    this.editForm = this.fb.group({
      oneMonthPrice: [null, [Validators.required, Validators.min(0)]],
      threeMonthsPrice: [null, [Validators.required, Validators.min(0)]],
      sixMonthsPrice: [null, [Validators.required, Validators.min(0)]],
      oneYearPrice: [null, [Validators.required, Validators.min(0)]]
    });
  }

  ngOnInit(): void {
    this.loadData();
  }

  ngOnDestroy(): void {
    this.destroy$.next(null);
    this.destroy$.complete();
  }

  loadData() {
    this.loading = true;
    this.cdr.detectChanges();
    this.priceService.pageOfColumn(this.searchParams)
      .pipe(takeUntil(this.destroy$))
      .subscribe({
        next: data => {
          this.loading = false;
          this.pageData = data;
        },
        error: data => {
          console.log(data)
          this.loading = false;
          this.message.error("获取专栏数据失败！");
        }
      });
  }

  changePaginationPage(page: number): void {
    this.searchParams['page'] = page - 1;
    this.cdr.detectChanges();
    this.loadData();
  }

  changePaginationSize(size: number): void {
    this.searchParams['page'] = 0;
    this.searchParams['size'] = size;
    this.cdr.detectChanges();
    this.loadData();
  }

  editPrice(columnPrice: ColumnPrice) {
    console.log('Editing column:', columnPrice);
    const modalRef = this.modal.create<SetColumnPriceComponent>({
      nzTitle: '编辑专栏价格',
      nzContent: SetColumnPriceComponent,
      nzData: columnPrice,
      nzFooter: null
    });

    modalRef.afterClose.subscribe(result => {
      if (result) {
        this.loadData();
      }
    });
  }

  // 将元转换为分
  private convertYuanToCents(yuan: number): number {
    return Math.round(yuan * 100);
  }
}
