import { Component, Inject } from '@angular/core';
import { FormBuilder, FormGroup, Validators, ReactiveFormsModule } from '@angular/forms';
import { NzFormModule } from 'ng-zorro-antd/form';
import { NzInputNumberModule } from 'ng-zorro-antd/input-number';
import { NzModalRef, NZ_MODAL_DATA } from 'ng-zorro-antd/modal';
import { ColumnPrice } from '../../../../core/models/price';
import { PriceService } from '../../price.service';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzButtonModule } from 'ng-zorro-antd/button';

@Component({
  selector: 'app-set-column-price',
  standalone: true,
  imports: [
    ReactiveFormsModule,
    NzFormModule,
    NzInputNumberModule,
    NzButtonModule
  ],
  templateUrl: './set-column-price.component.html',
  styleUrls: ['./set-column-price.component.scss']
})
export class SetColumnPriceComponent {
  editForm: FormGroup;
  column: ColumnPrice;

  // 价格格式化器
  yuanFormatter = (value: number): string => {
    return value ? `¥ ${value}` : '';
  };

  yuanParser = (value: string): string => {
    return value.replace(/[^0-9.]/g, '');
  };

  // 百分比格式化器
  percentFormatter = (value: number): string => {
    return value ? `${(value * 100).toFixed(0)}%` : '';
  };

  percentParser = (value: string): string => {
    return value.replace('%', '').trim();
  };

  constructor(
    private fb: FormBuilder,
    private modalRef: NzModalRef,
    private priceService: PriceService,
    private message: NzMessageService,
    @Inject(NZ_MODAL_DATA) data: ColumnPrice
  ) {
    this.column = data;
    const price = data?.price || {};
    
    this.editForm = this.fb.group({
      oneMonthPrice: [price?.oneMonthPrice ? price.oneMonthPrice / 100 : 0, [Validators.required, Validators.min(0)]],
      threeMonthsPrice: [price?.threeMonthsPrice ? price.threeMonthsPrice / 100 : 0, [Validators.required, Validators.min(0)]],
      sixMonthsPrice: [price?.sixMonthsPrice ? price.sixMonthsPrice / 100 : 0, [Validators.required, Validators.min(0)]],
      oneYearPrice: [price?.oneYearPrice ? price.oneYearPrice / 100 : 0, [Validators.required, Validators.min(0)]],
      firstMontDiscount: [price?.firstMontDiscount || 1, [Validators.required, Validators.min(0), Validators.max(1)]],
      discount: [price?.discount || 1, [Validators.required, Validators.min(0), Validators.max(1)]]
    });
  }

  handleOk(): void {
    if (this.editForm.valid) {
      const formValue = this.editForm.value;
      const priceData = {
        ...this.column.price,
        oneMonthPrice: Math.round(formValue.oneMonthPrice * 100),
        threeMonthsPrice: Math.round(formValue.threeMonthsPrice * 100),
        sixMonthsPrice: Math.round(formValue.sixMonthsPrice * 100),
        oneYearPrice: Math.round(formValue.oneYearPrice * 100),
        firstMontDiscount: formValue.firstMontDiscount,
        discount: formValue.discount
      };

      this.priceService.update(priceData).subscribe({
        next: () => {
          this.message.success('价格更新成功');
          this.modalRef.close(true);
        },
        error: (error) => {
          this.message.error('价格更新失败');
          console.error('Error updating price:', error);
        }
      });
    }
  }

  handleCancel(): void {
    this.modalRef.close();
  }
}
